@@ -83,32 +83,43 @@ void BrightnessControl::onIdleTimeout(int msec)
 
 void BrightnessControl::onProfileLoad()
 {
-    // This unparsable conditional block means: if the current profile is more
-    // conservative than the previous one and the current brightness is lower
-    // than the new profile
+    // if the current profile is more conservative than the previous one and the
+    // current brightness is lower than the new profile
     if (((m_currentProfile == "Battery" && m_lastProfile == "AC") ||
          (m_currentProfile == "LowBattery" && (m_lastProfile == "AC" || m_lastProfile == "Battery"))) &&
         m_defaultValue > brightness()) {
+
         // We don't want to change anything here
         qCDebug(POWERDEVIL) << "Not changing brightness, the current one is lower and the profile is more conservative";
     } else if (m_defaultValue >= 0) {
         QVariantMap args;
         args["Value"] = QVariant::fromValue((float)m_defaultValue);
+
+        // plugging in/out the AC is always explicit
+        if ((m_currentProfile == "AC" && m_lastProfile != "AC") ||
+            (m_currentProfile != "AC" && m_lastProfile == "AC")) {
+            args["Explicit"] = true;
+            args["Silent"] = true; // but we still don't want to show the OSD then
+        }
+
         trigger(args);
     }
 }
 
 void BrightnessControl::triggerImpl(const QVariantMap& args)
 {
+    int newBrightness = -1;
     if (args.contains("Step")) {
         backend()->setBrightnessStep(args["Step"].toInt());
     } else if ((QMetaType::Type) args["Value"].type() == QMetaType::Int) {
         backend()->setBrightnessValue(args["Value"].toInt());
+        newBrightness = brightnessPercent(args["Value"].toInt());
     } else {
         backend()->setBrightness(args["Value"].toFloat());
+        newBrightness = args["Value"].toFloat();
     }
-    if (args["Explicit"].toBool() && !args["Silent"].toBool()) {
-        BrightnessOSDWidget::show(brightness());
+    if (args["Explicit"].toBool() && !args["Silent"].toBool() && newBrightness > -1) {
+        BrightnessOSDWidget::show(newBrightness);
     }
 }
 
@@ -173,14 +184,18 @@ void BrightnessControl::setBrightnessSilent(int percent)
 
 void BrightnessControl::increaseBrightness()
 {
-    backend()->brightnessKeyPressed(BrightnessLogic::Increase);
-    BrightnessOSDWidget::show(brightness());
+    const int newBrightness = backend()->brightnessKeyPressed(BrightnessLogic::Increase);
+    if (newBrightness > -1) {
+        BrightnessOSDWidget::show(brightnessPercent(newBrightness));
+    }
 }
 
 void BrightnessControl::decreaseBrightness()
 {
-    backend()->brightnessKeyPressed(BrightnessLogic::Decrease);
-    BrightnessOSDWidget::show(brightness());
+    const int newBrightness = backend()->brightnessKeyPressed(BrightnessLogic::Decrease);
+    if (newBrightness > -1) {
+        BrightnessOSDWidget::show(brightnessPercent(newBrightness));
+    }
 }
 
 int BrightnessControl::brightnessValue() const
@@ -228,5 +243,15 @@ void BrightnessControl::setBrightnessStep(int step)
     trigger(args);
 }
 
+int BrightnessControl::brightnessPercent(float value) const
+{
+    const float maxBrightness = brightnessValueMax();
+    if (maxBrightness <= 0) {
+        return 0;
+    }
+
+    return qRound(value / maxBrightness * 100);
+}
+
 }
 }
