@@ -27,6 +27,7 @@
 #include "powerdevilbackendinterface.h"
 #include "powerdevilpolicyagent.h"
 #include "powerdevilprofilegenerator.h"
+#include "powerdevil_debug.h"
 
 #include "actions/bundled/suspendsession.h"
 
@@ -34,27 +35,28 @@
 #include <Solid/Device>
 #include <Solid/DeviceNotifier>
 
-#include <KDebug>
 #include <KIdleTime>
 #include <KLocalizedString>
 #include <KMessageBox>
 #include <KNotification>
 #include <KServiceTypeTrader>
-#include <KStandardDirs>
 
 // #include <KActivities/Consumer>
 
 #include <QtCore/QTimer>
 #include <QtDBus/QDBusConnection>
 #include <QtDBus/QDBusConnectionInterface>
+#include <QtDBus/QDBusServiceWatcher>
+
+#include <QDebug>
 
 namespace PowerDevil
 {
 
-Core::Core(QObject* parent/*, const KComponentData &componentData*/)
+Core::Core(QObject* parent)
     : QObject(parent)
     , m_backend(0)
-//     , m_applicationData(componentData)
+    , m_notificationsWatcher(nullptr)
     , m_criticalBatteryTimer(new QTimer(this))
 //     , m_activityConsumer(new KActivities::Consumer(this))
     , m_pendingWakeupEvent(true)
@@ -79,7 +81,7 @@ void Core::loadCore(BackendInterface* backend)
     m_backend = backend;
 
     // Async backend init - so that KDED gets a bit of a speed up
-    kDebug() << "Core loaded, initializing backend";
+    qCDebug(POWERDEVIL) << "Core loaded, initializing backend";
     connect(m_backend, SIGNAL(backendReady()), this, SLOT(onBackendReady()));
     connect(m_backend, SIGNAL(backendError(QString)), this, SLOT(onBackendError(QString)));
     m_backend->init();
@@ -90,7 +92,7 @@ void Core::loadCore(BackendInterface* backend)
 
 void Core::onBackendReady()
 {
-    kDebug() << "Backend is ready, KDE Power Management system initialized";
+    qCDebug(POWERDEVIL) << "Backend is ready, KDE Power Management system initialized";
 
     m_profilesConfig = KSharedConfig::openConfig("powermanagementprofilesrc", KConfig::CascadeConfig);
 
@@ -99,12 +101,7 @@ void Core::onBackendReady()
         // Generate defaults
         bool toRam = m_backend->supportedSuspendMethods() & PowerDevil::BackendInterface::ToRam;
         bool toDisk = m_backend->supportedSuspendMethods() & PowerDevil::BackendInterface::ToDisk;
-        if (ProfileGenerator::generateProfiles(toRam, toDisk, true) == ProfileGenerator::ResultUpgraded) {
-            // Notify the user
-            emitNotification("warningnot", i18n("Your Power Profiles have been updated to be used with the new KDE Power "
-                                                "Management System. You can tweak them or generate a new set of defaults from "
-                                                "System Settings."), "system-software-update");
-        }
+        ProfileGenerator::generateProfiles(toRam, toDisk);
         m_profilesConfig->reparseConfiguration();
     }
 
@@ -144,8 +141,20 @@ void Core::onBackendReady()
     m_criticalBatteryTimer->setInterval(30000);
     connect(m_criticalBatteryTimer, SIGNAL(timeout()), this, SLOT(onCriticalBatteryTimerExpired()));
 
-    // In 30 seconds (so we are sure the user sees eventual notifications), check the battery state
-    QTimer::singleShot(30000, this, SLOT(checkBatteryStatus()));
+    // wait until the notification system is set up before firing notifications
+    // to avoid them showing ontop of ksplash...
+    if (QDBusConnection::sessionBus().interface()->isServiceRegistered("org.freedesktop.Notifications")) {
+        onServiceRegistered(QString());
+    } else {
+        m_notificationsWatcher = new QDBusServiceWatcher("org.freedesktop.Notifications",
+                                                         QDBusConnection::sessionBus(),
+                                                         QDBusServiceWatcher::WatchForRegistration,
+                                                         this);
+        connect(m_notificationsWatcher, SIGNAL(serviceRegistered(QString)), this, SLOT(onServiceRegistered(QString)));
+
+        // ...but fire them after 30s nonetheless to ensure they've been shown
+        QTimer::singleShot(30000, this, SLOT(onNotificationTimeout()));
+    }
 
     // All systems up Houston, let's go!
     emit coreReady();
@@ -220,7 +229,7 @@ void Core::refreshStatus()
 
 void Core::reparseConfiguration()
 {
-    PowerDevilSettings::self()->readConfig();
+    PowerDevilSettings::self()->load();
     m_profilesConfig->reparseConfiguration();
 
     // Config reloaded
@@ -233,7 +242,7 @@ void Core::loadProfile(bool force)
 
     // Policy check
     if (PolicyAgent::instance()->requirePolicyCheck(PolicyAgent::ChangeProfile) != PolicyAgent::None) {
-        kDebug() << "Policy Agent prevention: on";
+        qCDebug(POWERDEVIL) << "Policy Agent prevention: on";
         return;
     }
 
@@ -244,9 +253,9 @@ void Core::loadProfile(bool force)
     if (activity.isEmpty()) {
         activity = "default";
     }
-    kDebug() << "We are now into activity " << activity;
+    qCDebug(POWERDEVIL) << "We are now into activity " << activity;
     KConfigGroup activitiesConfig(m_profilesConfig, "Activities");
-    kDebug() << activitiesConfig.groupList() << activitiesConfig.keyList();
+    qCDebug(POWERDEVIL) << activitiesConfig.groupList() << activitiesConfig.keyList();
 
     // Are we mirroring an activity?
     if (activitiesConfig.group(activity).readEntry("mode", "None") == "ActLike" &&
@@ -255,17 +264,17 @@ void Core::loadProfile(bool force)
         activitiesConfig.group(activity).readEntry("actLike", QString()) != "LowBattery") {
         // Yes, let's use that then
         activity = activitiesConfig.group(activity).readEntry("actLike", QString());
-        kDebug() << "Activity is a mirror";
+        qCDebug(POWERDEVIL) << "Activity is a mirror";
     }
 
     KConfigGroup activityConfig = activitiesConfig.group(activity);
-    kDebug() << activityConfig.groupList() << activityConfig.keyList();
+    qCDebug(POWERDEVIL) << activityConfig.groupList() << activityConfig.keyList();
 
     // See if this activity has priority
     if (activityConfig.readEntry("mode", "None") == "SeparateSettings") {
         // Prioritize this profile over anything
         config = activityConfig.group("SeparateSettings");
-        kDebug() << "Activity is enforcing a different profile";
+        qCDebug(POWERDEVIL) << "Activity is enforcing a different profile";
         profileId = activity;
     } else if (activityConfig.readEntry("mode", "None") == "ActLike") {
         if (activityConfig.readEntry("actLike", QString()) == "AC" ||
@@ -274,12 +283,12 @@ void Core::loadProfile(bool force)
             // Same as above, but with an existing profile
             config = m_profilesConfig.data()->group(activityConfig.readEntry("actLike", QString()));
             profileId = activityConfig.readEntry("actLike", QString());
-            kDebug() << "Activity is mirroring a different profile";
+            qCDebug(POWERDEVIL) << "Activity is mirroring a different profile";
         }
     } else {
         // It doesn't, let's load the current state's profile
         if (m_loadedBatteriesUdi.isEmpty()) {
-            kDebug() << "No batteries found, loading AC";
+            qCDebug(POWERDEVIL) << "No batteries found, loading AC";
             profileId = "AC";
         } else {
             // Compute the previous and current global percentage
@@ -287,18 +296,18 @@ void Core::loadProfile(bool force)
 
             if (backend()->acAdapterState() == BackendInterface::Plugged) {
                 profileId = "AC";
-                kDebug() << "Loading profile for plugged AC";
+                qCDebug(POWERDEVIL) << "Loading profile for plugged AC";
             } else if (percent <= PowerDevilSettings::batteryLowLevel()) {
                 profileId = "LowBattery";
-                kDebug() << "Loading profile for low battery";
+                qCDebug(POWERDEVIL) << "Loading profile for low battery";
             } else {
                 profileId = "Battery";
-                kDebug() << "Loading profile for unplugged AC";
+                qCDebug(POWERDEVIL) << "Loading profile for unplugged AC";
             }
         }
 
         config = m_profilesConfig.data()->group(profileId);
-        kDebug() << "Activity is not forcing a profile";
+        qCDebug(POWERDEVIL) << "Activity is not forcing a profile";
     }
 
     // Release any special inhibitions
@@ -326,7 +335,7 @@ void Core::loadProfile(bool force)
     // Check: do we need to change profile at all?
     if (m_currentProfile == profileId && !force) {
         // No, let's leave things as they are
-        kDebug() << "Skipping action reload routine as profile has not changed";
+        qCDebug(POWERDEVIL) << "Skipping action reload routine as profile has not changed";
 
         // Do we need to force a wakeup?
         if (m_pendingWakeupEvent) {
@@ -355,7 +364,7 @@ void Core::loadProfile(bool force)
             } else {
                 // Ouch, error. But let's just warn and move on anyway
                 //TODO Maybe Remove from the configuration if unsupported
-                kWarning() << "The profile " << profileId <<  "tried to activate"
+                qCWarning(POWERDEVIL) << "The profile " << profileId <<  "tried to activate"
                                 << actionName << "a non-existent action. This is usually due to an installation problem,"
                                 " or to a configuration problem, or simply the action is not supported";
             }
@@ -368,16 +377,16 @@ void Core::loadProfile(bool force)
 
     // Now... any special behaviors we'd like to consider?
     if (activityConfig.readEntry("mode", "None") == "SpecialBehavior") {
-        kDebug() << "Activity has special behaviors";
+        qCDebug(POWERDEVIL) << "Activity has special behaviors";
         KConfigGroup behaviorGroup = activityConfig.group("SpecialBehavior");
         if (behaviorGroup.readEntry("performAction", false)) {
             // Let's override the configuration for this action at all times
             ActionPool::instance()->loadAction("SuspendSession", behaviorGroup.group("ActionConfig"), this);
-            kDebug() << "Activity overrides suspend session action";
+            qCDebug(POWERDEVIL) << "Activity overrides suspend session action";
         }
 
         if (behaviorGroup.readEntry("noSuspend", false)) {
-            kDebug() << "Activity triggers a suspend inhibition";
+            qCDebug(POWERDEVIL) << "Activity triggers a suspend inhibition";
             // Trigger a special inhibition - if we don't have one yet
             if (!m_sessionActivityInhibit.contains(activity)) {
                 int cookie =
@@ -389,7 +398,7 @@ void Core::loadProfile(bool force)
         }
 
         if (behaviorGroup.readEntry("noScreenManagement", false)) {
-            kDebug() << "Activity triggers a screen management inhibition";
+            qCDebug(POWERDEVIL) << "Activity triggers a screen management inhibition";
             // Trigger a special inhibition - if we don't have one yet
             if (!m_screenActivityInhibit.contains(activity)) {
                 int cookie =
@@ -442,7 +451,7 @@ void Core::onDeviceAdded(const QString& udi)
                          "Please check your system configuration"));
     }
 
-    kDebug() << "A new battery was detected";
+    qCDebug(POWERDEVIL) << "A new battery was detected";
 
     m_batteriesPercent[udi] = b->chargePercent();
     if (b->chargeState() == Solid::Battery::NoCharge) {
@@ -462,9 +471,6 @@ void Core::onDeviceAdded(const QString& udi)
                              i18n("Additional Battery Added"),
                              i18n("All pending suspend actions have been canceled."));
     }
-
-    // So we get a "Battery is low" notification directly on system startup if applicable
-    emitBatteryChargePercentNotification(chargePercent, 100);
 }
 
 void Core::onDeviceRemoved(const QString& udi)
@@ -483,7 +489,7 @@ void Core::onDeviceRemoved(const QString& udi)
     disconnect(b, SIGNAL(chargeStateChanged(int,QString)),
                this, SLOT(onBatteryChargeStateChanged(int,QString)));
 
-    kDebug() << "An existing battery has been removed";
+    qCDebug(POWERDEVIL) << "An existing battery has been removed";
 
     m_batteriesPercent.remove(udi);
     m_batteriesCharged.remove(udi);
@@ -549,7 +555,7 @@ bool Core::emitBatteryChargePercentNotification(int currentPercent, int previous
 
 void Core::onAcAdapterStateChanged(PowerDevil::BackendInterface::AcAdapterState state)
 {
-    kDebug();
+    qCDebug(POWERDEVIL);
     // Post request for faking an activity event - usually adapters don't plug themselves out :)
     m_pendingWakeupEvent = true;
     loadProfile();
@@ -706,16 +712,59 @@ void Core::onResumingFromIdle()
     }
 }
 
+void Core::onNotificationTimeout()
+{
+    // cannot connect QTimer::singleShot directly to the other method
+    onServiceRegistered(QString());
+}
+
+void Core::onServiceRegistered(const QString &service)
+{
+    Q_UNUSED(service);
+
+    static bool notificationsReady = false;
+    if (notificationsReady) {
+        return;
+    }
+
+    // Now emit all the notifications
+    checkBatteryStatus();
+
+    bool emitted = false;
+    for(auto it = m_batteriesPercent.constBegin(); it != m_batteriesPercent.constEnd(); ++it) {
+        if (emitBatteryChargePercentNotification((*it), 100)) {
+            emitted = true;
+        }
+    }
+
+     // need to refresh status to prevent the notification from showing again when charge percentage changes
+    if (emitted) {
+        refreshStatus();
+    }
+
+    notificationsReady = true;
+
+    if (m_notificationsWatcher) {
+        delete m_notificationsWatcher;
+        m_notificationsWatcher = nullptr;
+    }
+}
+
 BackendInterface* Core::backend()
 {
     return m_backend;
 }
 
-bool Core::isLidClosed()
+bool Core::isLidClosed() const
 {
     return m_backend->isLidClosed();
 }
 
+bool Core::isLidPresent() const
+{
+    return m_backend->isLidPresent();
+}
+
 qulonglong Core::batteryRemainingTime() const
 {
     return m_backend->batteryRemainingTime();
